<?php

namespace Drupal\eca_flag\Plugin\ECA\Event;

use Drupal\eca\Plugin\ECA\Event\EventBase;
use Drupal\eca_content\Event\ContentEntityDelete;
use Drupal\eca_content\Event\ContentEntityInsert;
use Drupal\flag\Event\FlagEvents;
use Drupal\flag\Event\FlaggingEvent;
use Drupal\flag\Event\UnflaggingEvent;
use Drupal\flag\FlaggingInterface;

/**
 * Plugin implementation of the ECA Events for flag events.
 *
 * @EcaEvent(
 *   id = "flag",
 *   deriver = "Drupal\eca_flag\Plugin\ECA\Event\FlagEventDeriver",
 *   eca_version_introduced = "1.0.0"
 * )
 */
class FlagEvent extends EventBase {

  /**
   * {@inheritdoc}
   */
  public static function definitions(): array {
    return [
      'flag' => [
        'label' => 'Flag',
        'event_name' => FlagEvents::ENTITY_FLAGGED,
        'event_class' => FlaggingEvent::class,
      ],
      'unflag' => [
        'label' => 'Unflag',
        'event_name' => FlagEvents::ENTITY_UNFLAGGED,
        'event_class' => UnflaggingEvent::class,
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function buildEventData(): array {
    $event = $this->event;
    $data = [];
    $flagging = NULL;
    $flaggings = NULL;
    if ($event instanceof FlaggingEvent) {
      $flagging = $event->getFlagging();
    }
    elseif ($event instanceof UnflaggingEvent) {
      $flaggings = $event->getFlaggings();
    }
    elseif ($event instanceof ContentEntityInsert && $event->getEntity() instanceof FlaggingInterface) {
      $flagging = $event->getEntity();
    }
    elseif ($event instanceof ContentEntityDelete && $event->getEntity() instanceof FlaggingInterface) {
      $flaggings = [$event->getEntity()];
    }
    if ($flaggings !== NULL) {
      if (count($flaggings) === 1) {
        $flagging = reset($flaggings);
      }
      $flaggingTokens = [];
      foreach ($flaggings as $item) {
        $flaggingTokens[] = [
          'flag' => $item->getFlag(),
          'flagging' => $item,
          'entity' => $item->getFlaggable(),
        ];
      }
      $data['flagging'] = $flaggingTokens;
    }
    if ($flagging !== NULL) {
      $data += [
        'flag' => $flagging->getFlag(),
        'flagging' => $flagging,
        'entity' => $flagging->getFlaggable(),
      ];
      // Review the tokens when the issue #2500091 got completed in the flag
      // module.
      // @todo Also add the token for "flag-action"
      // @see https://www.drupal.org/project/flag/issues/2500091
    }

    $data += parent::buildEventData();
    return $data;
  }

}
